
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_MATH_HH_
#define _MOOF_MATH_HH_

/**
 * @file Math.hh
 * General math-related types and functions.
 */

#include <cmath>
#include <cml/cml.h>

#include <Moof/OpenGL.hh>		// GLscalar


namespace Mf {


typedef GLscalar								Scalar;

typedef cml::vector< Scalar, cml::fixed<2> >	Vector2;
typedef cml::vector< Scalar, cml::fixed<3> >	Vector3;
typedef cml::vector< Scalar, cml::fixed<4> >	Vector4;

typedef cml::matrix< Scalar, cml::fixed<2,2>,
		cml::col_basis, cml::col_major >		Matrix2;
typedef cml::matrix< Scalar, cml::fixed<3,3>,
		cml::col_basis, cml::col_major >		Matrix3;
typedef cml::matrix< Scalar, cml::fixed<4,4>,
		cml::col_basis, cml::col_major >		Matrix4;

typedef cml::quaternion< Scalar, cml::fixed<>, cml::vector_first,
		cml::positive_cross >					Quaternion;

typedef cml::constants<Scalar>					Constants;


inline Vector3 demote(const Vector4& vec)
{
	return Vector3(vec[0], vec[1], vec[2]);
}

inline Vector2 demote(const Vector3& vec)
{
	return Vector2(vec[0], vec[1]);
}

inline Vector4 promote(const Vector3& vec, Scalar extra = 1.0)
{
	return Vector4(vec[0], vec[1], vec[2], extra);
}

inline Vector3 promote(const Vector2& vec, Scalar extra = 1.0)
{
	return Vector3(vec[0], vec[1], extra);
}



const Scalar EPSILON = 0.000001;

/**
 * Check the equality of scalars with a certain degree of error allowed.
 */

inline bool isEqual(Scalar a, Scalar b, Scalar epsilon = EPSILON)
{
	return std::abs(a - b) < epsilon;
}


} // namespace Mf

#endif // _MOOF_MATH_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

